@@ -12,9 +12,9 @@
 // ```
 
 use glob::glob;
-use std::path::Path;
+use std::{fs, path::Path};
 
-fn main() {
+fn gen_tests() {
     let subplots = glob("*.md").expect("failed to find subplots in subplotlib");
     let tests = Path::new("tests");
     let subplots = subplots.chain(Some(Ok("../subplot.md".into())));
@@ -32,3 +32,11 @@ fn main() {
         subplot_build::codegen(Path::new(&entry)).expect("failed to generate code with Subplot");
     }
 }
+
+fn main() {
+    // Because we cannot generate tests if we're not fully inside the main subplot tree
+    // we only generate them if we can see ../subplot.md which is a good indicator.
+    if fs::metadata("../subplot.md").is_ok() {
+        gen_tests();
+    }
+}
