@@ -4,6 +4,8 @@
 //! Instead instances of it are constructed in the generated test functions and
 //! will be run automatically.
 
+use core::fmt;
+use std::fmt::Debug;
 use std::{cell::RefCell, marker::PhantomData, sync::Mutex};
 
 use state::Container;
@@ -18,7 +20,7 @@ use crate::types::{StepError, StepResult};
 /// be aware that they won't always be called on scenario start and they will
 /// not be caught up the first time they are invoked for a step, simply expected
 /// to get on with life from their first use.
-pub trait ContextElement: Default + Send + 'static {
+pub trait ContextElement: Debug + Default + Send + 'static {
     /// A new context element was created.
     ///
     /// In order to permit elements which for example work on disk, this
@@ -122,6 +124,9 @@ trait ScenarioContextHookKind {
 
     /// Leave a step
     fn step_stops(&self, contexts: &ScenarioContext) -> StepResult;
+
+    /// Produce your debug output
+    fn debug(&self, contexts: &ScenarioContext, dc: &mut DebuggedContext, alternate: bool);
 }
 
 impl<C> ScenarioContextHookKind for ScenarioContextHook<C>
@@ -143,6 +148,10 @@ where
     fn step_stops(&self, contexts: &ScenarioContext) -> StepResult {
         contexts.with_mut(|c: &mut C| c.step_stops(), true)
     }
+
+    fn debug(&self, contexts: &ScenarioContext, dc: &mut DebuggedContext, alternate: bool) {
+        contexts.with_generic(|c: &C| dc.add(c, alternate));
+    }
 }
 
 /// A container for all scenario contexts
@@ -154,6 +163,54 @@ pub struct ScenarioContext {
     hooks: RefCell<Vec<Box<dyn ScenarioContextHookKind>>>,
 }
 
+#[derive(Default)]
+struct DebuggedContext {
+    body: Vec<String>,
+}
+
+impl DebuggedContext {
+    fn add<C>(&mut self, obj: &C, alternate: bool)
+    where
+        C: Debug,
+    {
+        let body = if alternate {
+            format!("{:#?}", obj)
+        } else {
+            format!("{:?}", obj)
+        };
+        self.body.push(body);
+    }
+}
+
+struct DebugContextString<'a>(&'a str);
+
+impl<'a> Debug for DebugContextString<'a> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.write_str(self.0)
+    }
+}
+
+impl Debug for DebuggedContext {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_list()
+            .entries(self.body.iter().map(|s| DebugContextString(s)))
+            .finish()
+    }
+}
+
+impl Debug for ScenarioContext {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        let mut contexts = DebuggedContext::default();
+        for hook in self.hooks.borrow().iter() {
+            hook.debug(self, &mut contexts, f.alternate());
+        }
+        f.debug_struct("ScenarioContext")
+            .field("title", &self.title)
+            .field("contexts", &contexts)
+            .finish()
+    }
+}
+
 impl ScenarioContext {
     fn new(title: &str) -> Self {
         Self {
@@ -186,6 +243,22 @@ impl ScenarioContext {
         }
     }
 
+    fn with_generic<C, F>(&self, func: F)
+    where
+        F: FnOnce(&C),
+        C: ContextElement,
+    {
+        let sci: &ScenarioContextItem<C> = self
+            .inner
+            .try_get()
+            .expect("Scenario Context item not initialised");
+        let lock = match sci.0.lock() {
+            Ok(lock) => lock,
+            Err(pe) => pe.into_inner(),
+        };
+        func(&lock)
+    }
+
     /// With the extracted immutable context, run the function f.
     pub fn with<C, F, R>(&self, func: F, defuse_poison: bool) -> Result<R, StepError>
     where
@@ -224,6 +297,21 @@ impl ScenarioContext {
 /// Scenario objects are built up by the generated test functions and then run
 /// to completion.  In rare cases they may be built up and cached for reuse
 /// for example if a scenario is a subroutine.
+///
+/// Scenarios are built from steps in sequence, and then can be run.
+///
+/// ```
+/// # use subplotlib::prelude::*;
+///
+/// let mut scenario = Scenario::new("example scenario");
+///
+/// let run_step = subplotlib::steplibrary::runcmd::run::Builder::default()
+///     .argv0("true")
+///     .args("")
+///     .build("when I run true".to_string());
+/// scenario.add_step(run_step, None);
+///
+/// ```
 pub struct Scenario {
     contexts: ScenarioContext,
     steps: Vec<(ScenarioStep, Option<ScenarioStep>)>,
@@ -284,6 +372,7 @@ impl Scenario {
         // Firstly, we start all the contexts
         let mut ret = Ok(());
         let mut highest_start = None;
+        println!("scenario: {}", self.contexts.title());
         for (i, hook) in self.contexts.hooks.borrow().iter().enumerate() {
             let res = hook.scenario_starts(&self.contexts);
             if res.is_err() {
@@ -292,19 +381,25 @@ impl Scenario {
             }
             highest_start = Some(i);
         }
-
+        if ret.is_err() {
+            println!("*** Context hooks returned failure",);
+        }
         if ret.is_ok() {
             let mut highest = None;
             for (i, step) in self.steps.iter().map(|(step, _)| step).enumerate() {
+                println!("  step: {}", step.step_text());
                 let mut highest_prep = None;
                 for (i, prep) in self.contexts.hooks.borrow().iter().enumerate() {
-                    let res = prep.step_starts(&self.contexts, step.name());
+                    let res = prep.step_starts(&self.contexts, step.step_text());
                     if res.is_err() {
                         ret = res;
                         break;
                     }
                     highest_prep = Some(i);
                 }
+                if ret.is_err() {
+                    println!("*** Context hooks returned failure",);
+                }
                 if ret.is_ok() {
                     let res = step.call(&self.contexts, false);
                     if res.is_err() {
@@ -323,7 +418,11 @@ impl Scenario {
             if let Some(n) = highest {
                 for stepn in (0..=n).rev() {
                     if let (_, Some(cleanup)) = &self.steps[stepn] {
+                        println!("  cleanup: {}", cleanup.step_text());
                         let res = cleanup.call(&self.contexts, true);
+                        if res.is_err() {
+                            println!("*** Cleanup returned failure",);
+                        }
                         ret = ret.and(res);
                     }
                 }
@@ -336,6 +435,7 @@ impl Scenario {
                 ret = ret.and(res);
             }
         }
+        println!("  return: {}", if ret.is_ok() { "OK" } else { "Failure" });
         ret
     }
 }
